''***********************************************
''  GPS SPIN Object for GPS Receiver Module (#28506)   
''  Author: John Rucker
''***********************************************

''-----------------REVISION HISTORY-----------------
'' v1.0 - 11/6/2012 Creation date 


CON
  'Set Clock speed
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000
  'Screen formating
  CR=13, LF=10, CLS=13
  'Firmware version
  MajorVersion=1, MinorVersion=0
  'Pins
  gpsPin=0      'pin number the GPS receiver is connected to

OBJ
  gps           : "simple_serial"
  sNum          : "Simple_Numbers.spin"
  fp            : "FloatMath"
  fS            : "FloatString"
  
  '{'remove this line for standalone operation
  pst           : "Parallax Serial Terminal"  
  str           : "Strings2.spin"   
  '}'remove this line for standalone operation 
  
  
VAR
  Byte gpsHard, gpsFirm, gpsValid, gpsSats, gpsHours, gpsMinutes, gpsSeconds, gpsMonth, gpsDay, gpsYear
  Long gpsLat, gpsLong, gpsAltitude, gpsSpeed, gpsHead
  
  '{'remove this line for standalone operation
Pub Demo
  StandAloneDemo
  '}'remove this line for standalone operation

Pub InitializeGPS
  '***************************************
  ''Called to setup communicaitons with GPS
  ''Returns True if cog starts, False if not
  '***************************************
  result:=gps.init(gpsPin,gpsPin,9600)
  if result == False
    return False
  else
    return True 
     
Pub Latitude
  '***************************************
  ''Returns Floating Point Value of GPS Latitude in Dec format ie (39.1537)
  ''Call GetGpsLocaiton before calling this method to update value 
  '**************************************
  return gpsLat

Pub Longitude
  '***************************************
  ''Returns Floating Point Value of GPS Longitude in Dec format ie (-90.31902)
  ''Call GetGpsLocaiton before calling this method to update value   
  '**************************************
  return gpsLat

Pub Altitude
  '***************************************
  ''Returns Floating Point Value of GPS Altitude in Meters
  ''Call GetGpsLocaiton before calling this method to update value   
  '**************************************
  return gpsAltitude

Pub Speed
  '***************************************
  ''Returns Floating Point Value of GPS Speed in Knots
  ''Call GetGpsLocaiton before calling this method to update value   
  '**************************************
  return gpsSpeed

Pub Head
  '***************************************
  ''Returns Floating Point Value of GPS heading
  ''Call GetGpsLocaiton before calling this method to update value   
  '**************************************
  return gpsHead
  
Pub Valid
  '***************************************
  ''Returns Integer Value 0=Not Valid, 1=Valid
  ''Call GetGPSstats before calling this method to update value
  '**************************************
  return gpsValid

Pub Hours
  '***************************************
  ''Returns Integer Value representing hour of day in 24hr format (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value
  '**************************************
  return gpsHours

Pub Minutes
  '***************************************
  ''Returns Integer Value representing the minutes of the day (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value   
  '**************************************
  return gpsMinutes

Pub Seconds
  '***************************************
  ''Returns Integer Value representing Seconds of day (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value   
  '**************************************
  return gpsSeconds

Pub Month
  '***************************************
  ''Returns Integer Value representing Month of day (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value   
  '**************************************
  return gpsMonth

Pub Day
  '***************************************
  ''Returns Integer Value representing Day of the Month (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value    
  '**************************************
  return gpsDay

Pub Year
  '***************************************
  ''Returns Integer Value representing Year (UTC/Greenwich Mean Time)
  ''Call GetGPSstats before calling this method to update value     
  '**************************************
  return gpsYear

Pub LocalTime(TimeZone,Hour)|x 
  '***************************************
  ''Find local Time based on adding TimeZone to hour. Use negative number for western hemisphere.
  ''Returns correct hour of day based on TimeZone.
  '***************************************
  x := Hour + TimeZone
  if x < 0
    x := x + 24
  return x

Pub EstimateTimeZone(fLong) | x
  '***************************************
  ''Estimates local time Zone based on Longitude. fLong is afloating point long Longitude
  ''in gps decimal foramt (-90.31904 would return -6 as the estimated time zone)
  ''Returns signed Long Time Zone based on 15 degrees Longitude for each time zone.
  '***************************************
  x:=0
  x:=(fp.FRound(fLong) / 15)   'Divide by 15 to estimate local time zone
  return x

Pub GetGPSstats | x, Degree, Minute, fMinute, Direction
  '***************************************
  ''Read GPS non location data.
  ''Reads and sets several byte sized memory vales, see below for details 
  '***************************************
  gps.str(String("!GPS"))
  gps.Tx($00)                    'Get Version
  gpsHard:=gps.rx
  gpsFirm:=gps.rx
  
  gps.str(String("!GPS"))
  gps.tx($01)                    'Check for valid data string
  gpsValid:=gps.rx
  
  gps.str(String("!GPS"))
  gps.tx($02)                    'Get number of acquired satellites (20 max)
  gpsSats:=gps.rx
  
  gps.str(String("!GPS")) 
  gps.tx($03)                    'Get Time
  gpsHours:=gps.rx
  gpsMinutes:=gps.rx
  gpsSeconds:=gps.rx
  
  gps.str(String("!GPS")) 
  gps.tx($04)                   'Get Date
  gpsDay:=gps.rx  
  gpsMonth:=gps.rx
  gpsYear:=gps.rx

Pub GetGpsLocaiton | Deg, Mn, fMn, Dir
  '***************************************
  '' Read and sets GPS Latitude, Longitude, Altitude, Speed, Heading and convert the GPS' MinDec format to Decimal Degrees
  '' Sets long memeory values gpsLat and gpsLong to their Decimal Degrees in floating point format
  '***************************************
  gpsLat:=gpsLong:=gpsAltitude:=gpsSpeed:=gpsHead:=0
  'Find Latitude
  Deg:=Mn:=fMn:=Dir:=0
  gps.str(String("!GPS"))                               'VPN1513 GPS reports coordinates in MinDecimal format 399.2225N
  gps.tx($05)                                           'Get Latitude
  Deg:=gps.rx                                           'Degrees
  Mn:=gps.rx                                            'Minutes
  Byte[@fMn][1]:=gps.rx                                 'Fractional Minutes MSB
  Byte[@fMn][0]:=gps.rx                                 'Fractional Minutes LSB
  Dir:=gps.rx                                           'Direction 0=N 1=S
  gpsLat := gpsRawToMinDec(Deg,Mn,fMn,Dir)              'Convert GPS bytes to Floating Point Decimal Degrees
  'Find Longitude
  Deg:=Mn:=fMn:=Dir:=0
  gps.str(String("!GPS"))                               'VPN1513 GPS reports coordinates in MinDecimal format 399.2225N
  gps.tx($06)                                           'Get Longitude
  Deg:=gps.rx                                           'Degrees
  Mn:=gps.rx                                            'Minutes
  Byte[@fMn][1]:=gps.rx                                 'Fractional Minutes MSB
  Byte[@fMn][0]:=gps.rx                                 'Fractional Minutes LSB
  Dir:=gps.rx                                           'Direction 0=N 1=S
  gpsLong := gpsRawToMinDec(Deg,Mn,fMn,Dir)             'Convert GPS bytes to Floating Point Decimal Degrees
  'Find Altitude in tenths of meters
  gps.str(String("!GPS"))                               '6553.5 meters max value
  gps.tx($07)                                           'Get Altitude
  Byte[@gpsAltitude][1]:=gps.rx                         'GPS Altitude MSB
  Byte[@gpsAltitude][0]:=gps.rx                         'GPS Altitude LSB
  gpsAltitude:=TenthsToFloat(gpsAltitude)               'Convert to Floating Point
  'Find Speed in tenths of knots
  gps.str(String("!GPS"))                               '999.9 knots max value
  gps.tx($08)                                           'Get Speed
  Byte[@gpsSpeed][1]:=gps.rx                            'GPS Speed MSB
  Byte[@gpsSpeed][0]:=gps.rx                            'GPS Speed LSB
  gpsSpeed:=TenthsToFloat(gpsSpeed)                     'Convert to Floating Point
  'Find Heading (Direction of travel in tenths of degrees)
  gps.str(String("!GPS"))                               
  gps.tx($09)                                           'Get Heading
  Byte[@gpsHead][1]:=gps.rx                             'GPS Heading MSB
  Byte[@gpsHead][0]:=gps.rx                             'GPS Heading LSB
  gpsHead:=TenthsToFloat(gpsHead)                       'Convert to Floation Point
  
Pri TenthsToFloat(lValue)
  '***************************************
  ' Converts lValue that has imbedded tenths value to floating point value
  ' If lValue = 12345 result = 1234.5 as a floating point vlue
  '***************************************
  Result:=fp.FFloat(lValue)                             'Convert lValue to Float
  Result:=fp.FMul(Result,0.1)                           'Converts 12345 to 1234.5
  return Result
    
Pri gpsRawToMinDec(Degree,Minute,fMinute,Direction) : gpsDecimal | x
  '***************************************
  ' Degree = byte size degrees as read from GPS
  ' Minute = byte size Minutes as read from GPS
  ' fMinute = Word size Fractional Minutes from GPS
  ' Direction = byte size 1 or 0 as read from GPS
  ' Returns gps data (gpsDecimal) as a Long Floating Point
  '***************************************
      
  Degree:=fp.FFloat(Degree)                             'Convert Degree to Float
  Minute:=fp.FFloat(Minute)                             'Convert Minute to Floating Point
  fMinute:=fp.FFloat(fMinute)                           'Convert Farctional Minutes to Floating Point
  case strsize (fs.FloatToString(fMinute))              'Based on the number of digits in fMinute pick Multipler
    1: x:=0.1
    2: x:=0.01                                          
    3: x:=0.001
    4: x:=0.0001                                        
    5: x:=0.00001    
  fMinute:=fp.FMul(fMinute,x)                           'Convert Fractional Minutes to actual Floating point value
  gpsDecimal:=fp.FAdd(Minute,fMinute)                   'Add Minutes to Fractiona Minutes to get actual Floating Point
  gpsDecimal:=fp.FDiv(gpsDecimal, 60.0)                 'Divide by 60 to convert minutes into degree 
  gpsDecimal:=fp.FAdd(Degree,gpsDecimal)                'Add together to get Decimal Degrees

  if Direction == 1
    gpsDecimal:=fp.FMul(gpsDecimal,-1.0)                'Negate the result if South or  West 
  return gpsDecimal
  
  '{'remove this line for standalone operation   
Pub StandAloneDemo | x
  '***************************************
  ' Start PST Communications 
  '***************************************
  waitcnt (clkfreq * 3 + cnt) 
  pst.Start(250000)
  pst.clear    
  pst.Str(String("GPS Object Online. Version "))
  pst.Dec(MajorVersion)
  pst.str(String("."))
  pst.Dec(MinorVersion)

  x:=InitializeGPS 
  if x == False
    pst.Str(String(cr,"Error GPS Init"))
    abort
  else
    pst.Str(String(" GPS COG Started."))

  repeat
    GetGpsStats
    pst.Char(cr)
    repeat 85
      pst.STR(String("_"))
    pst.str(String(cr,"Date "))
    pst.dec(gpsMonth)
    pst.str(String("/"))
    pst.dec(gpsDay)
    pst.str(String("/"))
    pst.dec(gpsYear)
    pst.str(String(", Time "))
    pst.dec(gpsHours)
    pst.str(String(":"))
    pst.dec(gpsMinutes)
    pst.str(String(":"))
    pst.dec(gpsSeconds)         
    pst.Str(String(" GPS Ver= "))
    pst.dec(gpsHard)
    pst.Str(String("."))
    pst.dec(gpsFirm)
    if gpsValid == 1
      pst.str(String(", Data is Valid, "))
    else
      pst.str(String(", DATA NOT VALID. "))
    pst.dec(gpsSats)      
    pst.str(String(" Acquired Satellites"))
    GetGpsLocaiton
    pst.str(String(cr,"Lat "))
    pst.str(fs.FloatToString(gpsLat))
    pst.str(String(", Long "))
    pst.str(fs.FloatToString(gpsLong))
    pst.str(String(", Altitude "))
    pst.str(fs.FloatToString(gpsAltitude))
    pst.str(String("m, Speed "))
    pst.str(fs.FloatToString(gpsSpeed))
    pst.str(String("k, Heading "))
    pst.str(fs.FloatToString(gpsHead))
    pst.str(String(", Time "))
    pst.dec(LocalTime(EstimateTimeZone(gpsLong),gpsHours))
    pst.Char(":")
    pst.dec(gpsMinutes)
    repeat 60
      waitcnt(clkfreq+cnt)

  '}'remove this line for standalone operation   